/*
	Milyn - Copyright (C) 2003

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License (version 2.1) as published by the Free Software 
	Foundation.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    
	See the GNU Lesser General Public License for more details:    
	http://www.gnu.org/licenses/lgpl.txt
*/

package org.milyn.delivery.trans;

/**
 * Transformation Unit <b>Prototype</b> (GoF) Interface.
 * <p/>
 * TransUnit implementations should be stateless objects.  Statefull TransUnit implementations 
 * should be implemented through the TransUnitPrototype interface.  This provides the
 * Smooks framework with the clone method definition ({@link #cloneTransUnit}) needed to create 
 * multiple runtime instances of a statefull TransUnit.
 * <p/>
 * See <a href="../package.html>Delivery Overview</a>.
 * <p/>
 * @see org.milyn.delivery.trans.TransUnit
 * @see org.milyn.delivery.ContentDeliveryUnitCreator
 * @author tfennelly
 */
public interface TransUnitPrototype extends TransUnit {

	/**
	 * Clone this statefull TransUnit.
	 * @return TransUnit clone
	 */
	public TransUnit cloneTransUnit();
}
